// Copyright (c) 2013 by István Váradi

// This file is part of XPLRA, a remote-access plugin for X-Plane

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#ifndef XPLRA_GETDATAREFTASK_H
#define XPLRA_GETDATAREFTASK_H
//------------------------------------------------------------------------------

#include "DataRefTask.h"

//------------------------------------------------------------------------------

namespace xplra {

//------------------------------------------------------------------------------

/**
 * A dataref task which retrieves the value of a byte array.
 */
class GetByteArrayDataRefTask : public DataRefTask
{
private:
    /**
     * The maximal number of bytes accepted.
     */
    int maxBytes;

    /**
     * The offset within the dataref's value to start querying from.
     */
    int offset;

    /**
     * The array to store the data in.
     */
    unsigned char* data;

    /**
     * The actual length of the data retrieved.
     */
    int length;

public:
    /**
     * Construct the task for the dataref with the given name.
     *
     * @param maxBytes the maximal number of bytes to retrieve. If <0,
     * the function will perform an extra query to retrieve the size
     * of the dataref's data, which it will store in the task, so
     * later on it can be reused.
     */
    GetByteArrayDataRefTask(const std::string& name,
                            int maxBytes = -1, int offset = 0);

    /**
     * Construct the task for the given dataref
     *
     * @param maxBytes the maximal number of bytes to retrieve. If <0,
     * the function will perform an extra query to retrieve the size
     * of the dataref's data, which it will store in the task, so
     * later on it can be reused.
     */
    GetByteArrayDataRefTask(XPLMDataRef dataRef,
                            int maxBytes = -1, int offset = 0);


    /**
     * Destroy the task object.
     */
    virtual ~GetByteArrayDataRefTask();

    /**
     * Get the data.
     */
    const unsigned char* getData() const;

    /**
     * Get the length of the data.
     */
    int getLength() const;

protected:
    /**
     * Process the dataref by querying its value.
     */
    virtual void process();
};

//------------------------------------------------------------------------------
// Inline definitions
//------------------------------------------------------------------------------

inline const unsigned char* GetByteArrayDataRefTask::getData() const
{
    return data;
}

//------------------------------------------------------------------------------

inline int GetByteArrayDataRefTask::getLength() const
{
    return length;
}

//------------------------------------------------------------------------------

} /* namespace xplra */

//------------------------------------------------------------------------------
#endif // XPLRA_GETDATAREFTASK_H

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
