// Copyright (c) 2013 by István Váradi

// This file is part of XPLRA, a remote-access plugin for X-Plane

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

//------------------------------------------------------------------------------

#include "ServerThread.h"

#include "RequestQueue.h"
#include "GetDataRefTask.h"
#include "TaskRequest.h"

#include <xplcommon/Util.h>

#include <cstdio>

//------------------------------------------------------------------------------

using xplra::ServerThread;

using xplcommon::LocalAcceptor;
using xplcommon::Mutex;
using xplcommon::Util;

using std::string;

//------------------------------------------------------------------------------

Mutex ServerThread::instancesMutex;

ServerThread::instances_t ServerThread::instances;

//------------------------------------------------------------------------------

void ServerThread::quitAll()
{
    instancesMutex.lock();
    for(instances_t::iterator i = instances.begin(); i!=instances.end(); ++i) {
        (*i)->quit();
    }
    instancesMutex.unlock();
}

//------------------------------------------------------------------------------

ServerThread::ServerThread(RequestQueue& requestQueue, LocalAcceptor& acceptor) :
    Thread(true),
    requestQueue(requestQueue),
    bufferedStream(acceptor.getSocket(&waiter)),
    stream(*bufferedStream)
{
    instancesMutex.lock();
    instances.insert(this);
    instancesMutex.unlock();
}

//------------------------------------------------------------------------------

ServerThread::~ServerThread()
{
    delete bufferedStream;

    instancesMutex.lock();
    instances.erase(this);
    instancesMutex.unlock();
}

//------------------------------------------------------------------------------

void ServerThread::quit()
{
    stream.interrupt();
}

//------------------------------------------------------------------------------

void ServerThread::run()
{
    Util::debug("hu.varadiistvan.xplra.ServerThread[%p]::run\n", this);
    while(stream) {
        uint8_t command = stream.readU8();
        if (!stream) continue;

        // Util::debug("hu.varadiistvan.xplra.ServerThread[%p]::run: command=0x%02x\n",
        //             this, command);

        if (command==CMD_GET_SINGLE) {
            string name = stream.readString();
            uint8_t type = stream.readU8();
            if (!stream) continue;
            // Util::debug("hu.varadiistvan.xplra.ServerThread[%p]::run: name='%s', type=0x%02x\n",
            //             this, name.c_str(), type);
            if (type==TYPE_BYTE_ARRAY) {
                int length = stream.readS32();
                if (!stream) continue;

                // Util::debug("hu.varadiistvan.xplra.ServerThread[%p]::run: length=%lu\n",
                //             this, static_cast<unsigned long>(length));

                GetByteArrayDataRefTask* task =
                    new GetByteArrayDataRefTask(name, length);
                TaskRequest request(task);

                if (requestQueue.execute(&request)) {
                    // Util::debug("hu.varadiistvan.xplra.ServerThread[%p]::run: request executed\n");
                    stream.writeU8(RESULT_OK);
                    int length = task->getLength();
                    stream.writeS32(length);
                    if (length>0) {
                        stream.write(task->getData(), length);
                    }
                } else {
                    break;
                }
            } else {
                stream.writeU8(RESULT_OTHER_ERROR);
            }
        } else {
            stream.writeU8(RESULT_INVALID_COMMAND);
        }
        stream.flush();
    }

    Util::debug("hu.varadiistvan.xplra.ServerThread[%p]::run: quitting\n", this);
}

//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
